"""
File: arrays.py

An Array is a restricted list whose clients can use
only [], len, iter, str, insert, and remove.

To instantiate, use

<variable> = array(<capacity>, <optional fill value>)

The fill value is None by default.
"""

class Array(object):
    """Represents an array."""

    def __init__(self, capacity, fillValue = None):
        """Capacity is the static size of the array.
        fillValue is placed at each position."""
        self._items = list()
        for count in xrange(capacity):
            self._items.append(fillValue)
        
        if fillValue == None:
            self._logicalSize = 0
        else:
            self._logicalSize = capacity
        self._initialCapacity = capacity
        self._fillValue = fillValue

    def size(self):
        """Returns the logical capacity of the array."""
        return self._logicalSize

    def grow(self):
        """Doubles the physical capacity of the array"""
        currentSize = len(self._items)
        for count in xrange(currentSize):
            self._items.append(self._fillValue)

    def insert(self, index, newItem):
        """Adds newItem at spot index of array or at the logical end if index > size().
           Precondition:  0 <= index"""
        if index < 0:
            raise IndexError, "Index out of range"

        if self._logicalSize == len(self._items):        # array full
            self.grow()
            
        if index >= self._logicalSize:                 
            self._items[self._logicalSize] = newItem
        else:
            # make room for newItem at index
            for i in xrange(self._logicalSize, index, -1):
                self._items[i] = self._items[i-1]
            self._items[index] = newItem
        self._logicalSize += 1

    def shrink(self):
        """Halve the physical capacity of the array down to the initial capacity"""
        # ADD CODE HERE
            
    def remove(self, index):
        """Removes and returns the item at spot index in the Array.  If
           the array drops below a quarter full, then its physical size is halved.
           Precondition:  0 <= index < size()"""
        # ADD CODE HERE

    def __eq__(self, rhsArray):
        """Returns True the two arrays have the same number of logical items and
           pairs of items from each logical position in the two arrays are equal.
           Otherwise, returns False.
           Precondition:  """
        # ADD CODE HERE
            
    def __len__(self):
        """-> The physical capacity of the array."""
        return len(self._items)

    def __str__(self):
        """-> The string representation of the array."""
        return str(self._items)

    def __iter__(self):
        """Supports traversal with a for loop."""
        return iter(self._items)

    def __getitem__(self, index):
        """Subscript operator for access at index.
           Precondition:  """
        return self._items[index]

    def __setitem__(self, index, newItem):
        """Subscript operator for replacement at index.
           Precondition:  """
        self._items[index] = newItem


myArray = Array(3)
print "myArray.size() = ", myArray.size()
myArray.insert(10, 'a')
print myArray
myArray.insert(0, 'b')
print myArray
myArray.insert(1, 'c')
print myArray
myArray.insert(4, 'd')
print myArray
print "myArray.size() = ", myArray.size()
print "len(myArray)", len(myArray)

